#!/bin/bash

. /usr/local/share/libubox/jshn.sh

def_cfg=`uci get gateway.general.gwcfg`
subband=`uci get gateway.general.subband`
gwid=`uci get gateway.general.GWID`
latitude=`uci get gateway.general.LAT`
longitude=`uci get gateway.general.LON`
altitude=`uci get gateway.general.ALT`
fake_gps=`uci get gateway.general.fake_gps`

chip="302"

gen_gw_cfg() {
	json_init
    	json_add_object gateway_conf
		json_add_string "platform" "SX1$chip"
		json_add_string "description" "Dragino LoRaWAN Gateway"
		json_add_string "gateway_ID" "$gwid"
		json_add_boolean "radiostream_enable" 1
		json_add_string "ghost_host" "localhost"
		json_add_string "ghost_port" "1760"
		json_add_boolean "manage_enable" 0
		json_add_boolean "custom_downlink" 0
		json_add_int	"stat_interval" "30"
		json_add_boolean "wd_enable" 1
		json_add_boolean "mac_decode" 0
		json_add_boolean "mac2file" 0
		json_add_boolean "mac2db" 0
		json_add_string "log_mask" "1111"
		json_add_double "ref_latitude" "$latitude"
		json_add_double "ref_longitude" "$longitude"
		json_add_double "ref_altitude" "$altitude"
		json_add_int "beacon_period" "0"
		json_add_int "beacon_freq_hz" "869525000"
		json_add_int "beacon_datarate" "9"
		json_add_int "beacon_bw_hz" "125000"
		json_add_int "beacon_power" "14"
		json_add_int "beacon_infodesc" "0"

		json_add_array "servers"
		json_add_object "server1"
			json_add_string "server_name" "myminer"
			json_add_boolean "enabled" 1
			json_add_string "server_type" "semtech"
			json_add_string	"server_address" "`uci get gateway.server1.server_address`"
			json_add_int "serv_port_up" "`uci get gateway.server1.upp`"
			json_add_int "serv_port_down" "`uci get gateway.server1.dpp`"
			json_add_int "keepalive_interval" "`uci get gateway.server1.keepalive_interval`"
			json_add_int "stat_interval" "30"
			json_add_int "push_timeout_ms" "`uci get gateway.server1.push_timeout_ms`"
			json_add_int "fport_filter" "`uci get gateway.server1.fport_filter`"
			json_add_int "devaddr_filter" "`uci get gateway.server1.devaddr_filter`"
			json_add_boolean "forward_crc_valid" "`uci get gateway.server1.forward_crc_valid`"
			json_add_boolean "forward_crc_error" "`uci get gateway.server1.forward_crc_error`"
			json_add_boolean "forward_crc_disabled" "`uci get gateway.server1.forward_crc_disabled`"
		json_close_object
	json_close_object
	json_close_array

	json_add_object debug_conf
	json_add_array "ref_payload"

		json_add_object
		json_add_string "id" "0xCAFE1234"
		json_close_object

		json_add_object
		json_add_string "id" "0xCAFE2345"
		json_close_object

	json_close_array
	json_add_string "log_file" "loragw_hal.log"
	json_close_object
	json_dump  > /etc/lora/local_conf.json.tmp
	jq . /etc/lora/local_conf.json.tmp   >  /etc/lora/local_conf.json
	#json_dump  > /root/test1
}

gen_cus_cfg() {
	json_init
		json_add_object SX130x_conf
			json_add_string "spidev_path" "/dev/spidev1.0"
			json_add_boolean "lorawan_public" 1			
			json_add_int "clksrc" "0"
			json_add_int "antenna_gain" "0"
			json_add_boolean "full_duplex" 0
			json_add_object precision_timestamp
			json_add_boolean "enable" 0
			json_add_int "max_ts_metrics" "255"
			json_add_int "nb_symbols" "1"
			json_close_object

			json_add_object radio_0
			json_add_boolean "enable" "`uci get gateway.general.radio0_enable`"
			json_add_string "type" "SX1250"
			radio0_freq="`uci get gateway.general.radio0_freq`"
			json_add_int "freq" "$radio0_freq"
			json_add_double "rssi_offset" "-215.4"
			json_add_object rssi_tcomp
				json_add_int "coeff_a" "0"
				json_add_int "coeff_b" "0"
				json_add_double "coeff_c" "20.41"
				json_add_double "coeff_d" "2162.56"
				json_add_int "coeff_e" "0"
			json_close_object
			json_add_boolean "tx_enable" "`uci get gateway.general.radio0_tx`"
			json_add_int "tx_freq_min" "`uci get gateway.general.radio0_txfreq_min`"
			json_add_int "tx_freq_max" "`uci get gateway.general.radio0_txfreq_max`"

			json_add_array "tx_gain_lut"
				for a in `seq 12 14`
				do
					c=$[$a + 3]
					json_add_object
						json_add_int "rf_power" "$a"
						json_add_int "pa_gain" "0"
						json_add_int "pwr_idx" "$c"
					json_close_object
				done
			
			json_add_object
				json_add_int "rf_power" "15"
				json_add_int "pa_gain" "0"
				json_add_int "pwr_idx" "19"
			json_close_object

			json_add_object
				json_add_int "rf_power" "16"
				json_add_int "pa_gain" "0"
				json_add_int "pwr_idx" "20"
			json_close_object

			json_add_object
				json_add_int "rf_power" "17"
				json_add_int "pa_gain" "0"
				json_add_int "pwr_idx" "22"
			json_close_object

			for b in `seq 18 24`
			do
				c=$[$b - 17]
				json_add_object
					json_add_int "rf_power" "$b"
					json_add_int "pa_gain" "1"
					json_add_int "pwr_idx" "$c"
				json_close_object
			done

			json_add_object
				json_add_int "rf_power" "25"
				json_add_int "pa_gain" "1"
				json_add_int "pwr_idx" "9"
			json_close_object
			
			json_add_object
				json_add_int "rf_power" "26"
				json_add_int "pa_gain" "1"
				json_add_int "pwr_idx" "11"
			json_close_object
			
			json_add_object
				json_add_int "rf_power" "27"
				json_add_int "pa_gain" "1"
				json_add_int "pwr_idx" "14"
			json_close_object
			json_close_array
			json_close_object

			json_add_object radio_1
				json_add_boolean "enable" "`uci get gateway.general.radio1_enable`"
				json_add_string "type" "SX1250"
				radio1_freq="`uci get gateway.general.radio1_freq`"
				json_add_int "freq" "$radio1_freq"
				json_add_double "rssi_offset" "-215.4"

				json_add_object rssi_tcomp
					json_add_int "coeff_a" "0"
					json_add_int "coeff_b" "0"
					json_add_double "coeff_c" "20.41"
					json_add_double "coeff_d" "2162.56"
					json_add_int "coeff_e" "0"
				json_close_object
				json_add_boolean "tx_enable" "0"
			json_close_object
			
			for i in `seq 0 7`
			do
			json_add_object chan_multiSF_$i
				json_add_boolean "enable" "`uci get gateway.general.chan${i}_enable`"
				chan_radio="`uci get gateway.general.chan${i}_radio`"
				json_add_int "radio" "$chan_radio"
				chan_if="`uci get gateway.general.chan$i`"
				json_add_int "if" "$chan_if"
				radio0_freq="`uci get gateway.general.radio0_freq`"
				radio1_freq="`uci get gateway.general.radio1_freq`"
				 if [ "${chan_radio}" = "0" ]; then
					json_add_string "desc" "Lora MAC, 125kHz, all SF, ${radio0_freq}, if: ${chan_if}"
				 elif [ "${chan_radio}" = "1" ]; then
					json_add_string "desc" "Lora MAC, 125kHz, all SF, ${radio1_freq}, if: ${chan_if}"
				 fi
			json_close_object
			done
		
			json_add_object chan_Lora_std
				json_add_boolean "enable" "`uci get gateway.general.lorachan_enable`"
				chan_radio="`uci get gateway.general.lorachan_radio`"
				json_add_int "radio" "$chan_radio"
				chan_if="`uci get gateway.general.lorachan`"
				json_add_int "if" "$chan_if"
				chan_band="`uci get gateway.general.lorachan_bw`"
				json_add_int "bandwidth" "$chan_band"
				chan_sf="`uci get gateway.general.lorachan_sf`"
				json_add_int "spread_factor" "$chan_sf"
				json_add_boolean "implicit_hdr" "0"
				json_add_int "implicit_payload_length" "17"
				json_add_boolean "implicit_crc_en" "0"
				json_add_int "implicit_coderate" "1"
				if [ "$chan_radio" = "0" ]; then
					json_add_string "desc" "Lora MAC, SF=$chan_sf, bandwidth=$chan_band, $radio0_freq, if: $chan_if"
				elif [ "$chan_radio" = "1" ]; then
					json_add_string "desc" "Lora MAC, SF=$chan_sf, bandwidth=$chan_band, $radio1_freq, if: $chan_if"
				fi
			json_close_object
		
			json_add_object chan_FSK
				json_add_boolean "enable" "1"
				json_add_int "radio" "1"
				json_add_int "if" "300000"
				json_add_int "bandwidth" "125000"
				json_add_int "daterate" "50000"
				json_add_string "desc" "FSK 50kbps, 868.8 MHz"
			json_close_object
		json_close_object

			/ * gwid="`uci get gateway.general.GWID`"
			json_add_object "gateway_conf"
				json_add_string "gateway_ID" "$gwid"
				json_add_string "server_address" "localhost"
				json_add_int "serv_port_up" "1700"
				json_add_int "serv_port_down" "1700"
				json_add_int "keepalive_interval" "10"
				json_add_int "stat_interval" "30"
				json_add_int "push_timeout_ms" "100"
				json_add_boolean "forward_crc_valid" "1"
				json_add_boolean "forward_crc_error" "0"
				json_add_boolean "forward_crc_disabled" "0"
				json_add_string "gps_tty_path" "/dev/ttyS0"
				json_add_int "ref_latitude" "0"
				json_add_int "ref_longitude" "0"
				json_add_int "ref_altitude" "0"
				json_add_int "beacon_period" "0"
				json_add_int "beacon_freq_hz" "869525000"
				json_add_int "beacon_datarate" "9"
				json_add_int "beacon_bw_hz" "125000"
				json_add_int "beacon_power" "14"
				json_add_int "beacon_infodesc" "0"
			json_close_object * /

			json_add_object "debug_conf"
				json_add_array "ref_payload"
					json_add_object
						json_add_string "id" "0xCAFE1234"
					json_close_object
					json_add_object
						json_add_string "id" "0xCAFE2345"
					json_close_object
				json_close_array
			json_add_string "log_file" "loragw_hal.log"
			json_close_object
	
			json_dump  > /etc/lora/global_conf.json				
			
}

echo_chan_if()
{
	echo "Gateway Channels Frequnecy" > /etc/lora/desc
	echo "---------------------------------------" >> /etc/lora/desc

	json_load_file "/etc/lora/global_conf.json"
	json_select SX130x_conf
	for i in `seq 0 7`
	do
		json_select chan_multiSF_$i
		json_get_var desc desc
		json_select ..
		echo "chan_multSF_$i" >> /etc/lora/desc
		echo "$desc" >> /etc/lora/desc
		echo "---------------------------------------" >> /etc/lora/desc
	 done
	
	json_select chan_Lora_std
  	json_get_var desc desc
	json_select ..
	echo "chan_Lora_std" >> /etc/lora/desc
	echo "$desc" >> /etc/lora/desc
	echo "---------------------------------------" >> /etc/lora/desc

	 json_select chan_FSK
	json_get_var desc desc
	json_select ..
	echo "chan_FSK" >> /etc/lora/desc
	echo "$desc" >> /etc/lora/desc
	echo "---------------------------------------" >> /etc/lora/desc
	
	json_cleanup


}

gen_gb_cfg()
{
	if [ "$def_cfg" = "AU" ] || [ "$def_cfg" = "US" ] || [ "$def_cfg" = "CN" ];then
	def_cfg="$def_cfg-$subband"
	fi
	if [ -f /etc/lora/cfg-$chip/"$def_cfg"-global_conf.json ]; then 
   	 	cp -rf /etc/lora/cfg-$chip/"$def_cfg"-global_conf.json /etc/lora/global_conf.json
	elif [ "$def_cfg" = "CUS" ];then
		logger "Customized Freq"
		gen_cus_cfg
	else 
    		cp -rf /etc/lora/cfg-$chip/EU-global_conf.json /etc/lora/global_conf.json
	fi

}

gwcfg="$(uci -q get gateway.general.gwcfg)"

case "$1" in
	start)
		if [ "$def_cfg" = "CUS" ]; then
			gen_cus_cfg
		else
			gen_gw_cfg # Generate local_conf.json
		fi
		gen_gb_cfg # Generate global_conf.json
		echo_chan_if # Generate the used frequnecy

		;;
	*)
esac

exit
